require 'tk'

#
# #########################################################
#
# For displaying error messages.
#
require 'Win32API'

#
# Button styles:
#
$MB_OK = 0
$MB_OKCANCEL = 1
$MB_ABORTRETRYIGNORE = 2
$MB_YESNOCANCEL = 3
$MB_YESNO = 4
$MB_RETRYCANCEL = 5

#
# Icon styles:
#
$MB_ICONHAND = 16
$MB_ICONQUESTION = 32
$MB_ICONEXCLAMATION = 48
$MB_ICONASTERISK = 64

#
# Default button:
#
$MB_DEFBUTTON1 = 0
$MB_DEFBUTTON2 = 256
$MB_DEFBUTTON3 = 512

#
# Return values:
#
$IDOK = 1
$IDCANCEL = 2
$IDABORT = 3
$IDRETRY = 4
$IDIGNORE = 5
$IDYES = 6
$IDNO = 7

def messageBox(message, title = 'Message', style = 0)
	mb = Win32API.new("user32", "MessageBox", ['L', 'P', 'P', 'L'], 'L')
	return mb.Call(0, message, title, style)
end

#
# #########################################################
#
# Window drawing section.
#

font_name = "arial 10"

root = TkRoot.new() { title "" }
root.geometry('+460+650')

top_frame = TkFrame.new(root).pack(:side=>:top, :fill=>:x)
bot_frame = TkFrame.new(root).pack(:side=>:top, :fill=>:x)

state_box_label = TkLabel.new(top_frame, :text=>"Check Box Control", :relief=>:sunken, :background=>"lightgrey", "font"=>"arial 10").pack(:fill=>:x)

state_text = TkVariable.new
state_entry = TkEntry.new(top_frame, :textvariable=>state_text, :width=>13, "font"=>"arial 10").pack(:pady=>5)
state_entry.insert(0, "box")
state_entry.focus

get_method_button = TkButton.new(bot_frame, :width=>14, :text=>"Retrieve Method", :background=>"lightgrey", :foreground=>"blue", "font"=>font_name).pack(:pady=>5, :padx=>5)

cancel_button = TkButton.new(bot_frame, :width=>14, :text=>"Cancel Operation", :background=>"lightgrey", :foreground=>"blue", "font"=>font_name).pack(:pady=>5, :padx=>5)

#
# ###############################################################
#
# Methods:
#
def process_errors(error, var)

	err_msg = [
	"  ERROR - #{var} is not a valid box format - enter like a regular box.\n\n\tExample: box1_1\n"
	]

	#
	# Using bitwise operations, error is set on a bit by bit basis, where each bit represents an error message.
	#
	if (error > 0)

		mask = 1

		#
		# Math.frexp(error) returns a two element array containing the normalized fraction (a Float) and exponent
		# (a Fixnum) of error. In our case, the fraction is 0.5 and the exponent is a power of 2, as our error is 1 x 2
		# raised to 0,1,2,3,... So this returns 0.5 x 2[a power]. So, say error = 4, which is 1 x 2[2], which is the 3rd
		# error in our list. .frexp returns [0.5, 3] - we want the 3 [3rd error], we use 3 - 1 since we are starting
		# from 0 not 1. Just doing this so that I do not have to keep updating the 0.upto( ) each time I add a new error.
		#
		0.upto(Math.frexp(error)[1]-1) do |shift|
			if (((mask << shift) & error) > 0)
				messageBox(err_msg[shift], "Error!", $MB_ICONEXCLAMATION|$MB_OK)
			end
		end
	end
end

#
# ###############################################################
#
# Button Bindings:
#
cancel_button.bind("Button-1") { exit(0) }

get_method_button.bind("Button-1") {

	#
	# This outputs one_check_box_at_a_time.rbw in response to a SciTE Tools command.
	#
	if (state_text.value =~ /^box\d+_\d+$/)

		print "\n"
		File.open("//PDC/FormsProgramming/Shared Code/one_check_box_at_a_time.rbw") do |file|
			file.each_line { |line| print line.gsub("state_box", state_text.value)}
		end
	
		exit(0)
	else
		process_errors(1, state_text.value)
	end
}

#
# ###############################################################
#
Tk.mainloop()
